package eu.etaxonomy.taxeditor.prototype1.view;

import java.text.MessageFormat;
import java.util.Set;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.map.IMapChangeListener;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.core.databinding.observable.map.MapChangeEvent;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import eu.etaxonomy.taxeditor.prototype1.view.NameEditorView.Node;

/**
 * @author tomson <tom.schindl@bestsolution.at>
 * @version $Revision: $ 
 * @since 1.0
 *
 */
public class GenericObservableMapCellLabelProvider extends ColumnLabelProvider {
	/**
	 * Template text
	 */
	private String templateText;

	/**
	 * The attribute maps
	 */
	private final IObservableMap[] attributeMaps;

	/**
	 * Change listener to track changes
	 */
	private IMapChangeListener mapChangeListener = new IMapChangeListener() {
		public void handleMapChange(MapChangeEvent event) {
			Set affectedElements = event.diff.getChangedKeys();
			LabelProviderChangedEvent newEvent = new LabelProviderChangedEvent(
					GenericObservableMapCellLabelProvider.this, affectedElements.toArray());
			fireLabelProviderChanged(newEvent);
		}
	};

	private DataBindingContext bc;

	/**
	 * @param attributeMap
	 * @param templateText 
	 */
	public GenericObservableMapCellLabelProvider(IObservableMap attributeMap,
			String templateText, DataBindingContext bc) {
		this(new IObservableMap[] { attributeMap },templateText, bc);
	}

	/**
	 * @param attributeMaps
	 * @param templateText 
	 */
	public GenericObservableMapCellLabelProvider(IObservableMap[] attributeMaps,String templateText, DataBindingContext bc) {
		this.attributeMaps = attributeMaps;
		for (int i = 0; i < attributeMaps.length; i++) {
			attributeMaps[i].addMapChangeListener(mapChangeListener);
		}
		this.templateText = templateText;
		this.bc = bc;
	}

	public void dispose() {
		for (int i = 0; i < attributeMaps.length; i++) {
			attributeMaps[i].removeMapChangeListener(mapChangeListener);
		}
		super.dispose();
	}

	public Image getImage(Object element) {
		return null;
	}

	public String getText(Object element) {
		return ((Node) element).getValue();		
	}
	
	public void update(ViewerCell cell) {
				
		Object element = cell.getElement();
 
		cell.setText(getText(element));
		Image image = getImage(element);
		cell.setImage(image);
		cell.setBackground(getBackground(element));
		cell.setForeground(getForeground(element));
		cell.setFont(getFont(element));

		// bind model element to its corresponding cell 
		IObservableValue observeNodeValue = BeansObservables.observeValue((Node) element, "value");
		IObservableValue observeCellValue = BeansObservables.observeValue(cell, "text");
		bc.bindValue(observeNodeValue, observeCellValue, null, null);
		
	}
}