/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.preference.InitNomenclaturalCodePrefDialog;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/** 
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {
	private static final Logger logger = Logger.getLogger(ApplicationWorkbenchWindowAdvisor.class);
	
	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	public ActionBarAdvisor createActionBarAdvisor(
			IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(963, 637));
		configurer.setShowCoolBar(true);
		configurer.setShowStatusLine(true);
		configurer.setShowPerspectiveBar(true);
		configurer.setTitle("EDIT Taxonomic Editor");
	}
	
	public void postWindowOpen() {
		checkNomenclaturalCode();
	}
	
	private void checkNomenclaturalCode() {
		
		// First time Editor is opened, no nomenclatural code has been set
		if (PreferencesUtil.getPreferredNomenclaturalCode() == null) {

			// Query user re: preferred nom. code
			Dialog dialog = new InitNomenclaturalCodePrefDialog(UiUtil.getShell());
			int returnCode = dialog.open();
			
			// Short message confirming user's choice
			String code = PreferencesUtil.getPreferredNomenclaturalCodeAsString();
			MessageDialog.openInformation(UiUtil.getShell(), "Nomenclatural code set", 
					"The following has been set as your preferred nomenclatural code:\n\n\t" + 
					code + "\n\nYou can change the botanical code at any time in the \"Preferences\" menu.");
		}		
	}
	
}
