/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.cdm;

/**
 * @author p.ciardelli
 * @created 14.05.2008
 * @version 1.0
 */
import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonOperation;

/**
 * Delete this taxon from the CDM
 * 
 * @author p.ciardelli
 * 
 */
public class DeleteTaxonAction extends Action {
	private static final Logger logger = Logger
			.getLogger(DeleteTaxonAction.class);

	private static String text = "Delete taxon from tree";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);

	private IOperationHistory operationHistory;
	private IUndoContext undoContext;
	
	Taxon taxon;

	public DeleteTaxonAction() {
		super(text);
		setImageDescriptor(image);
		
		operationHistory = UiUtil.getOperationHistory();
		undoContext = UiUtil.getWorkbenchUndoContext();
	}

	public DeleteTaxonAction(Taxon taxon) {
		this();

		this.taxon = taxon;
	}

	public void run() {
		IUndoableOperation operation = new DeleteTaxonOperation
				(this.getText(), undoContext, taxon);

		IProgressMonitor monitor = UiUtil.getStatusLineProgressMonitor();
		
		try {
			IStatus status = operationHistory.execute(operation, monitor, null);
		} catch (ExecutionException e) {
			logger.error("Error deleting taxon", e);
		}	
		
		// Handle this in an operation so we are able to undo it
		
//		
//		// Get taxon name for progress monitor
//		String taxonName = CdmUtil.getDisplayName(taxon);
//		
//		// Get and start progress monitor
//		IProgressMonitor monitor = UiUtil.getStatusLineProgressMonitor();
//		monitor.beginTask("Deleting taxon '" + taxonName + "'.", 10);
//		
//		// Call save all action
//		new SaveAllAction().run(null);
//		monitor.worked(2);
//		
//		// If the taxon has children, cancel operation
//		// TODO add option to continue, and delete children
//		if (taxon.hasTaxonomicChildren()) {
//			MessageDialog.openInformation(UiUtil.getShell(), "Cannot delete taxon", 
//					"'" + CdmUtil.getDisplayName(taxon) + "' has taxonomic children. " +
//					"These must be manually deleted before their parent.");
//			monitor.done();
//			return;
//		}
//		monitor.worked(1);
//		
//		
//		// Close taxon's editor, if any is active
//		try {
//			UiUtil.closeEditor(taxon, true);
//		} catch (PartInitException e) {
//			e.printStackTrace();
//		}
//		monitor.worked(1);
//
//		// Delete taxon from CDM layer
//		CdmSessionDataRepository.getDefault().deleteTaxon(taxon);
//		monitor.worked(4);
//				
//		
//		
//		// Call save all action
//		new SaveAllAction().run(null);
//		monitor.worked(2);
//		
//		// Close the progress monitor
//		monitor.done();
//		
//		// Announce taxon was deleted on status line
//		UiUtil.setStatusLine("Deleted taxon '" + taxonName + "'.");
//		
////		Status status = new Status(IStatus.INFO, TaxEditorPlugin.PLUGIN_ID, 
////				"Deleted.", null);
////		StatusManager.getManager().handle(status, StatusManager.SHOW);
	}
}