/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.MakeGroupBasionymAction;
import eu.etaxonomy.taxeditor.editor.name.Messages;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class AddBasionymCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AddBasionymCompositeAction.class);

	private String text;
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.BASIONYM_ICON);

	private TaxonNameBase basionymName;

	private NameComposite composite;
	private IUndoableOperation operation;

	private AddBasionymCompositeAction() {
		super();
		setImageDescriptor(image);
	}

	public AddBasionymCompositeAction(NameComposite composite) {
		this();
		
		if (composite.getData() instanceof Synonym) {
			Synonym synonym = (Synonym) composite.getData();
			this.basionymName = synonym.getName();
		} else {
			throw new IllegalArgumentException(
					"This action requires a composite with a Synonym in its data field."); //$NON-NLS-1$
		}

		text = Messages.getString("AddBasionymCompositeAction.0", basionymName); //$NON-NLS-1$
		setText(text);
		
		this.composite = composite;

		operation = new AddBasionymOperation();
		
		if (CdmUtil.isNameGroupBasionym(basionymName)) {
			setEnabled(false);
		} else {
			setEnabled(true);			
		}
	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class AddBasionymOperation extends AbstractOperation {

		public AddBasionymOperation() {
			super("'" + text + "'"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			Action action = new MakeGroupBasionymAction(basionymName);
			action.addPropertyChangeListener(new IPropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent event) {
					if (event.getProperty().equals(
							ITaxEditorConstants.BASIONYM)) {
						composite.transform(NameComposite.ADD_GROUP_BASIONYM);
						setEnabled(false);
						firePropertyChange(ITaxEditorConstants.BASIONYM, null, basionymName);
					}
				}
			});			
			action.run();
			
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}
	}
}
