/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.UndoableAction;
import eu.etaxonomy.taxeditor.actions.cdm.SwapSynonymAndTaxonAction;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;

/**
 * @author p.ciardelli
 * @created 09.06.2008
 * @version 1.0
 */
public class ChangeSynonymToTaxonUiAction extends UndoableAction {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToTaxonUiAction.class);

	private static String text = "Make synonym this taxon's accepted taxon";
	private static ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.SWAP_SYNONYM_AND_TAXON_ICON);

	private NameComposite composite;
	private Taxon taxon;
	private Synonym synonym;

	public ChangeSynonymToTaxonUiAction(Synonym synonym, Taxon taxon) {
		super(text, image);

		this.synonym = synonym;
		this.taxon = taxon;
	}

	@Override
	protected IStatus executeOperation() {
		Action action = new SwapSynonymAndTaxonAction(synonym, taxon);
		action.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(ITaxEditorConstants.TAXON)) {

					if (event.getNewValue() instanceof Taxon) {
						Taxon newTaxon = (Taxon) event.getNewValue();

						// Close editor for existing taxon
						IEditorPart oldEditor;
						try {
							oldEditor = UiUtil.getEditorByTaxon(taxon);
							UiUtil.closeEditor(oldEditor, true);
						} catch (PartInitException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						// Remove old taxon from tree
						CdmSessionDataRepository.getDefault().removeTaxon(taxon);

						// Add new taxon to tree
						CdmSessionDataRepository.getDefault().addTaxon(newTaxon);

						// Open editor for new taxon
						new OpenTaxonEditorAction(newTaxon).run();

						// Remove old taxon from recent names list
						RecentNamesView.addRecentName(taxon);
					}

				}
			}
		});
		action.run();
		return Status.OK_STATUS;
	}

	@Override
	protected IStatus redoOperation() {
		return Status.OK_STATUS;
	}

	@Override
	protected IStatus undoOperation() {
		return Status.OK_STATUS;
	}
}
