/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.editor.SelectTaxonDialog;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;

/**
 * @author p.ciardelli
 * @created 26.05.2008
 * @version 1.0
 */
public class ChangeTaxonToSynonymAction extends Action {
	private static final Logger logger = Logger
			.getLogger(ChangeTaxonToSynonymAction.class);

	private static String text = "Change this taxon to a synonym";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageRegistry().getDescriptor(ITaxEditorConstants.TAXON_TO_SYNONYM_ICON);

	private Taxon oldTaxon;

	private ChangeTaxonToSynonymAction() {
		super(text);
		setImageDescriptor(image);
	}

	public ChangeTaxonToSynonymAction(Taxon oldTaxon) {
		this();
		this.oldTaxon = oldTaxon;
	}

	public void run() {
		
		IEditorPart oldEditor = null;
		
		try {	
			// Prompt user "Would you like to save?" before showing dialog
			// "Cancel" cancels action - "No" does not
			oldEditor = UiUtil.getEditorByTaxon(oldTaxon);
			if (oldEditor.isDirty()) {
				if (!MessageDialog.openConfirm(UiUtil.getShell(), "Save before proceeding", 
						"All changes must be saved before proceeding with this action.\n\n" +
						"Press \"OK\" to save and continue, or \"Cancel\" to cancel this action.")) {
					return;
				}
				oldEditor.doSave(null);
			}
//			if (UiUtil.getActivePage().saveEditor(oldEditor, true) == false) {
//				return;
//			}
		} catch (PartInitException e1) {
			e1.printStackTrace();
		}
		
		// Get destination taxon from dialog
		Shell shell = UiUtil.getShell();
		SelectTaxonDialog dialog = new SelectTaxonDialog(shell, SelectTaxonDialog.TAXON_TO_SYNONYM);
		Taxon destinationTaxon = dialog.open(oldTaxon);
		
		// Abort action if user cancelled dialog without choosing a taxon
		if (destinationTaxon == null) {
			return;
		}
				
		// Move oldTaxon in CDM
		TaxonNameBase synonymName = oldTaxon.getName();
		SynonymRelationshipType synonymType;
		if (CdmUtil.isNameHomotypic(synonymName, destinationTaxon)) {
//			makeTaxonSynonym
//			oldTaxon.makeSynonym();
//			destinationTaxon.addHomotypicSynonymName(synonymName);
			synonymType = SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF();
		} else {
//			destinationTaxon.addHeterotypicSynonymName(synonymName);
			synonymType = SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF();
		}
		CdmUtil.makeTaxonSynonym(oldTaxon, destinationTaxon, synonymType,
				null, null);
		
		// TODO move any children, descriptions, synonyms
		for (TaxonDescription description: oldTaxon.getDescriptions()) {
			destinationTaxon.addDescription(description);
			oldTaxon.removeDescription(description);
		}
		
		for (TaxonRelationship fromRelation : oldTaxon.getRelationsFromThisTaxon()) {
			fromRelation.setFromTaxon(destinationTaxon);
		}
		
		for (TaxonRelationship toRelation : oldTaxon.getRelationsToThisTaxon()) {
			toRelation.setToTaxon(destinationTaxon);
		}
				
//		if (!TaxEditorPlugin.getDefault().getObservableRecentNamesList().
//				contains(oldTaxon)) {	
//			TaxEditorPlugin.getDefault().getObservableRecentNamesList().
//					remove(oldTaxon);
//			RecentNamesView.removeRecentName(oldTaxon);
//		}
		
		CdmSessionDataRepository.getDefault().removeTaxon(oldTaxon);	
				
		// Close open editor for oldTaxon without forcing save.
		// User has already saved or declined to do so above.
		try {
			UiUtil.closeEditor(oldEditor, false);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
		// Open editor for new oldTaxon
		new SaveTaxonAction(destinationTaxon).run();
		new OpenTaxonEditorAction(destinationTaxon).run();
	}
}