/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;

/**
 * Looks for a EditorGroupComposite on an IManagedForm which has a value set for
 * composite.getData(ITaxEditorConstants.MISAPPLIED_NAME); if not found, creates
 * it.
 * 
 * Then puts the DescriptionElementComposite onto the misapplied name EditorGroupComposite. 
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class MoveCompositeToMisappliedCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(MoveCompositeToMisappliedCompositeAction.class);

	private static String text = "Change synonym to misapplied name";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);

	private IUndoableOperation operation;
	private NameComposite composite;
	private IManagedForm managedForm;

	private MoveCompositeToMisappliedCompositeAction() {
		super(text);
		setImageDescriptor(image);
	}

	public MoveCompositeToMisappliedCompositeAction(NameComposite composite,
			IManagedForm managedForm) {
		this();

		this.composite = composite;
		this.managedForm = managedForm;

		operation = new MisappliedNameOperation();

	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class MisappliedNameOperation extends AbstractOperation {

		public MisappliedNameOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			
			Composite groupComposite = UiUtil.getMisappliedNameGroupComposite(managedForm);
			if (groupComposite == null) {
				groupComposite = UiUtil.createMisappliedNameGroupComposite(managedForm);
			}
			composite.setParent(groupComposite);
						
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return execute(monitor, info);
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return Status.OK_STATUS;
		}
	}
}