/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui.description;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;

/**
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class DeleteElementCompositeAction extends Action {
	private static final Logger logger = Logger
			.getLogger(DeleteElementCompositeAction.class);

	private static String text = "Delete description element";
	private ImageDescriptor image = TaxEditorPlugin.getDefault()
			.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);

	private IUndoableOperation operation;
	private DescriptionElementBase descriptionElement;
	private Taxon taxon;

	private Composite elementComposite;
	private Composite featureComposite;
	private Composite descriptionComposite;
	private DescriptionBase description;

	private DeleteElementCompositeAction() {
		super(text);
		setImageDescriptor(image);
	}

	public DeleteElementCompositeAction(Composite elementComposite, 
			Taxon taxon) {
		this();

		this.elementComposite = elementComposite;
		
		if (elementComposite.getData() instanceof DescriptionElementBase) {
			this.descriptionElement = (DescriptionElementBase) elementComposite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires an elementComposite with a DescriptionElementBase in its data field.");
		}
		this.taxon = taxon;
		
		featureComposite = elementComposite.getParent();
		descriptionComposite = featureComposite.getParent();
		if (descriptionComposite.getData() instanceof DescriptionBase) {
			this.description = (DescriptionBase) descriptionComposite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires that the element composite's \"grandparent\" has a Description in its data field.");
		}
		operation = new DeleteElementOperation();
	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class DeleteElementOperation extends AbstractOperation {

		public DeleteElementOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			
			// Delete element from description
			description.removeElement(descriptionElement);
			
			// Dipose element composite
			elementComposite.dispose();
			
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			// TODO Auto-generated method stub
			return Status.OK_STATUS;
		}
	}
}
