/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.ui.DeleteMisappliedNameCompositeAction;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.operations.name.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.name.ChangeMisappliedNameToSynonymOperation;
import eu.etaxonomy.taxeditor.operations.name.ChangeSynonymToMisappliedNameOperation;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class MisappliedNameComposite extends NameComposite {
	private static final Logger logger = Logger
			.getLogger(MisappliedNameComposite.class);
	
	private Taxon misappliedName;
	
	public MisappliedNameComposite(Composite parent, IManagedForm form,
			Taxon taxon, Taxon misappliedName) {
		super(parent, form, NameComposite.MISAPPLIED_NAME, misappliedName);
		
		this.taxon = taxon;
		this.misappliedName = misappliedName;
		
		setDraggable(true);
		setIcon(MISAPPLIEDNAME_ICON);
		setFont(MISAPPLIEDNAME_FONT);
		setIndent(MISAPPLIEDNAME_INDENT);

		createMenu();
	}
	
	public Taxon getMisappliedName() {
		return misappliedName;
	}
	
	private void createMenu() {

		ContextMenu contextMenu = createContextMenu();

		Action deleteMisappliedNameAction = new DeleteMisappliedNameCompositeAction(
				this, getTaxon());
		contextMenu.addAction(deleteMisappliedNameAction);
	}
	
	public boolean setParent(Composite parent) {

		if (super.setParent(parent)) {	
			
			// If this has been moved to the misapplied names group, do nothing
			if (parent instanceof MisappliedGroupComposite) {
				return true;
			}

			// Has this been moved to a HomotypicalGroup?
			if (parent instanceof HomotypicalGroupComposite) {
				
				HomotypicalGroup homotypicalGroup = 
						((HomotypicalGroupComposite)parent).getGroup();
				
				IOperationHistory operationHistory = UiUtil.getOperationHistory();
				IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(taxon);
				IUndoableOperation operation = new ChangeMisappliedNameToSynonymOperation
						("change type", undoContext, taxon, misappliedName, homotypicalGroup);
				
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			return true;

		}
		return false;
	}
}
