/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.WidgetTransfer;
import eu.etaxonomy.taxeditor.actions.ui.AdaptCompositeToGroupAction;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.FreeTextElementFactory;
import eu.etaxonomy.taxeditor.editor.INameEditorCompositeRepository;
import eu.etaxonomy.taxeditor.operations.name.ChangeHomotypicGroupOperation;

/**
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class TaxonNameEditor extends AbstractTaxonEditor 
		implements INameEditorCompositeRepository{
	private static final Logger logger = Logger.getLogger(TaxonNameEditor.class);
		
	/**
	 * Shared listener that sets dirty state to true 
	 *  when any registered property changes 
	 */
	private PropertyChangeListener taxonChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getPropertyName().equals(ITaxEditorConstants.PROPERTY_SHEET_CHANGE)) {
				firePropertyChange(PROP_DIRTY);
			}
		}
	};		
	
	@Override
	public void doSave(IProgressMonitor monitor) {}

	@Override
	public void doSaveAs() {}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
				
		super.init(site, input);
		
		Taxon taxon = getTaxon();
	
		UiUtil.addTaxonNameEditor(taxon, this);
		
		// Register listeners for any change in accepted name or set of relations
		taxon.getName().addPropertyChangeListener(taxonChangeListener);
		taxon.addPropertyChangeListener(taxonChangeListener);
	}

	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}
	
	@Override
	public void createPartControl(final Composite composite) {

		super.createPartControl(composite);
		
		Taxon taxon = getTaxon();
				
		NameComposite acceptedNameComposite = 
				(NameComposite) FreeTextElementFactory.getDefault().
						createAcceptedTaxon(managedForm, taxon);
				
		for (TaxonBase taxonBase : new IterableSynonymyList(taxon)) {
			if (taxonBase instanceof Synonym) {
				FreeTextElementFactory.getDefault().createSynonym((Synonym) taxonBase, taxon);				
			} else {
				FreeTextElementFactory.getDefault().createMisappliedName((Taxon) taxonBase, taxon);
			}
		}
		
//		// Create a homotypic group composite for the accepted taxon
//		HomotypicalGroup homotypicGroup = taxon.getHomotypicGroup();
//		EditorGroupComposite homotypicGroupComposite = new EditorGroupComposite(parent, managedForm, homotypicGroup);	
//		homotypicGroupComposite.setData(ITaxEditorConstants.TAXON, taxon);
//		
//		NameComposite nameCompositeWithWarning = null;
//		
//		// Add accepted taxon to the group
//		NameComposite acceptedNameComposite = new NameComposite(homotypicGroupComposite, managedForm, NameComposite.ACCEPTED_TAXON, taxon);
//		
//		warnIfNameConflictWithParent(acceptedNameComposite);
//		
//		IterableSynonymyList synonymyList = new IterableSynonymyList(taxon);
//		HomotypicalGroup lastHeterotypicGroup = null;
//		HomotypicalGroup currentHeterotypicGroup = null;
//		EditorGroupComposite heterotypicGroupComposite = null;
//		EditorGroupComposite misappliedNameGroupComposite = null;
//				
//		for (TaxonBase synonymOrMisName : synonymyList) {
//			if (synonymOrMisName instanceof Synonym) {
//				
//				Synonym synonym = (Synonym) synonymOrMisName;
//				currentHeterotypicGroup = synonym.getHomotypicGroup();
//				
//				if (currentHeterotypicGroup.equals(homotypicGroup)) {
//					
//					NameComposite homotypicSynonymComposite = new NameComposite(homotypicGroupComposite, managedForm, NameComposite.HOMOTYPIC_SYNONYM, synonym);
//					
//				} else {
//					
//					if (!currentHeterotypicGroup.equals(lastHeterotypicGroup)) {
//						
//						// Create a new group composite
//						heterotypicGroupComposite = new EditorGroupComposite(parent, managedForm, currentHeterotypicGroup);
//						heterotypicGroupComposite.setData(ITaxEditorConstants.TAXON, taxon);
//						lastHeterotypicGroup = currentHeterotypicGroup;
//					}
//					
//					NameComposite heterotypicSynonymComposite = new NameComposite(heterotypicGroupComposite, managedForm, NameComposite.HETEROTYPIC_SYNONYM, synonym);
//				}
//			} else {
//				
//				Taxon misappliedName = (Taxon) synonymOrMisName;
//				if (UiUtil.getMisappliedNameGroupComposite(managedForm) == null) {
//					misappliedNameGroupComposite = UiUtil.createMisappliedNameGroupComposite(managedForm);
//				}
//				NameComposite misappliedNameComposite = new NameComposite(misappliedNameGroupComposite, managedForm, NameComposite.MISAPPLIED_NAME, misappliedName);
//			}
//		}
			
		// Listen for names being dragged outside of existing homotypic groups -
		//  user wants to create a new group
		Transfer[] types = new Transfer[] {WidgetTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		DropTarget target = new DropTarget(parent, operations);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			
			public void drop(DropTargetEvent event) {
				
				IOperationHistory operationHistory = UiUtil.getOperationHistory();
				
				Synonym synonym = null;
				if(event.data instanceof SynonymComposite){
					synonym = ((SynonymComposite)event.data).getSynonym();
				}
				
				IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(getTaxon());
				IUndoableOperation operation = new ChangeHomotypicGroupOperation
						("change type", undoContext, 
								getTaxon(), synonym, HomotypicalGroup.NewInstance());
				
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}		
		});
				
		// We've added elements to the managed form, so redraw it
		scrolledForm.reflow(true);
		
		setSelection(acceptedNameComposite);
		acceptedNameComposite.drawBorder();
	}
	
	
	
	public boolean redraw(){
		
		Taxon taxon = getTaxon();
		
		for (Control child : parent.getChildren()){
			child.dispose();
		}
		
		this.selectedObject = null;
		
		NameComposite acceptedNameComposite = 
				(NameComposite) FreeTextElementFactory.getDefault().
						createAcceptedTaxon(managedForm, taxon);
				
		for (TaxonBase taxonBase : new IterableSynonymyList(taxon)) {
			if (taxonBase instanceof Synonym) {
				FreeTextElementFactory.getDefault().createSynonym((Synonym) taxonBase, taxon);				
			} else {
				FreeTextElementFactory.getDefault().createMisappliedName((Taxon) taxonBase, taxon);
			}
		}
		
		scrolledForm.reflow(true);
				
		return true;
	}
	
	private NameComposite warnIfNameConflictWithParent(NameComposite nameComposite) {
		
		String message = null;
		Taxon taxon = getTaxon();
		
		if (nameComposite.getData() instanceof Taxon) {
			taxon = (Taxon) nameComposite.getData();
			Taxon parentTaxon = taxon.getTaxonomicParent();
			
			if (parentTaxon != null && taxon.getName() instanceof NonViralName 
					&& parentTaxon.getName() instanceof NonViralName) {
				NonViralName name = (NonViralName) taxon.getName();
				NonViralName parentName = (NonViralName) parentTaxon.getName();
				
				Rank rank = name.getRank();
				if (rank != null) {
					if (rank.equals(Rank.SPECIES())) {
						String genus = name.getGenusOrUninomial();
						String parentGenus = parentName.getGenusOrUninomial();

						// If either taxon has a problem, name fields will be null
						if (genus != null && parentGenus != null) {
							if (!parentGenus.equals(genus)) {
								message = "Warning: taxon's genus does not match parent taxon's genus.";
							}
						}
					}
	
					if (rank.equals(Rank.SUBSPECIES())) {
						String specificEpithet = name.getSpecificEpithet();
						String parentSpecificEpithet = parentName.getSpecificEpithet();
						
						// If either taxon has a problem, name fields will be null
						if (specificEpithet != null && parentSpecificEpithet != null) {
							if (!parentSpecificEpithet.equals(specificEpithet)) {
								message = "Warning: taxon's specific epithet does not match parent taxon's specific epithet.";
							}
						}
					}
				}
				
			}
		}
		
		if (message != null && nameComposite.getTextViewer() != null) {
			managedForm.getMessageManager().addMessage
					("TEST2", message, null, IMessageProvider.WARNING, 
							nameComposite.getTextViewer().getTextWidget());
			scrolledForm.setMessage(null, 0, null);
		}
		return nameComposite;
	}

	
	private HashSet<Composite> getAllComposites(){
		HashSet<Composite> composites = new HashSet<Composite>();
		composites.add(parent);
		composites.addAll(getComposites(parent));
		return composites;
	}
	
	private HashSet<Composite> getComposites(Composite composite){
		HashSet<Composite> composites = new HashSet<Composite>();
		for(Control child : composite.getChildren()){
			if(child instanceof Composite){
				composites.add((Composite)child);
				composites.addAll(getComposites((Composite)child));
			}
		}
		return composites;
	}
	
	
	@Override
	public Composite getAcceptedName() {
		
		for(Composite c : getAllComposites()){
			if(c instanceof AcceptedNameComposite){
				return c;
			}
		}
		return null;
	}

	@Override
	public Composite getHomotypicGroup(HomotypicalGroup group) {
		for(Composite c : getAllComposites()){
			if(c instanceof HomotypicalGroupComposite){
				if (group.equals(
						((HomotypicalGroupComposite) c).getGroup())) {
					return c;
				}
			}
		}
		return null;
	}

	@Override
	public Composite getMisappliedGroup() {
		for(Composite c : getAllComposites()){
			if(c instanceof MisappliedGroupComposite){
				return c;
			}
		}
		return null;
	}

	@Override
	public Composite getMisappliedName(Taxon misappliedName) {
		for(Composite c : getAllComposites()){
			if(c instanceof MisappliedNameComposite){
				if (misappliedName.equals(
						((MisappliedNameComposite) c).getMisappliedName())) {
					return c;
				}
			}
		}
		return null;
	}

	@Override
	public Composite getSynonym(Synonym synonym) {
		for(Composite c : getAllComposites()){
			if(c instanceof SynonymComposite){
				if (synonym.equals 
						(((SynonymComposite) c).getSynonym())) {
					return c;
				}
			}
		}
		return null;
	}
}