/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 29.05.2008
 * @version 1.0
 */
public class TaxonomicTreeLabelProvider extends ObservableMapLabelProvider {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeLabelProvider.class);
	
	public TaxonomicTreeLabelProvider(IObservableSet observableTaxonSet) {
		super(BeansObservables.observeMaps(
				observableTaxonSet, 
				Taxon.class, new String[]{"name"}));
	}
	
	/** 
	 * 
	 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnImage(java.lang.Object, int)
	 **/
	public Image getColumnImage(Object element, int columnIndex) {
		if (element instanceof Taxon) {
			TaxonNameBase name = ((Taxon) element).getName();
			if (name != null && name.getHasProblem()) {
				return TaxEditorPlugin.getDefault().getImage(ITaxEditorConstants.WARNING_ICON);
			}
		}
		return null;
	}
	/**
	 * JFace databinding syntax makes it to difficult to retrieve
	 * 	Taxon.getName().getNameCache, so override function that returns
	 *  label
	 * 
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnText(java.lang.Object, int)
	 **/
	public String getColumnText(Object element, int columnIndex) {
		if (element instanceof Taxon) {
			return CdmUtil.getDisplayName((Taxon) element);					
		}
		return "";
	}			

}
