/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * @author n.hoffmann
 * @created 16.01.2009
 * @version 1.0
 */
public class DeleteTaxonOperation extends AbstractEditorOperation {
	private static Logger logger = Logger.getLogger(DeleteTaxonOperation.class);

	private Taxon parentTaxon;
	public DeleteTaxonOperation(String text, IUndoContext undoContext,
			Taxon taxon) {
		super(text, undoContext, taxon);

		parentTaxon = taxon.getTaxonomicParent();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// TODO move this logic to another place
		
		// Get taxon name
		String taxonName = taxon.getName().getTitleCache();
		
		// Prompt user for confirmation
		if(! MessageDialog.openConfirm(UiUtil.getShell(), "Confirm Deletion", "Are you sure you want to delete taxon '" + taxonName + "'?")){
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		
		// Get and start progress monitor
		monitor.beginTask("Deleting taxon '" + taxonName + "'.", 10);
		
		// Call save all
		UiUtil.saveAll();
		monitor.worked(2);
		
		// If the taxon has children, cancel operation
		// TODO add option to continue, and delete children
		if (taxon.hasTaxonomicChildren()) {
			MessageDialog.openInformation(UiUtil.getShell(), "Cannot delete taxon", 
					"'" + taxonName + "' has taxonomic children. " +
					"These must be manually deleted before their parent.");
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		monitor.worked(1);
		
		
		// Close taxon's editor, if any is active
		try {
			UiUtil.closeEditor(taxon, true);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		monitor.worked(1);

		// Delete taxon from CDM layer
		CdmSessionDataRepository.getDefault().removeTaxon(taxon);
		monitor.worked(4);
				
		// Call save all
		UiUtil.saveAll();
		monitor.worked(2);
		
		// Close the progress monitor
		monitor.done();
		
		// Announce taxon was deleted on status line
		UiUtil.setStatusLine("Deleted taxon '" + taxonName + "'.");
		
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// FIXME we have to add old citation
		parentTaxon.addTaxonomicChild(taxon, null, null);
		CdmSessionDataRepository.getDefault().addTaxon(taxon);
		return Status.OK_STATUS;
	}
}
