/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

public class ChangeHomotypicGroupOperation extends AbstractEditorOperation {
	
	private static final Logger logger = Logger
			.getLogger(ChangeHomotypicGroupOperation.class);

	private Synonym synonym; 
	private HomotypicalGroup oldHomotypicalGroup;
	private HomotypicalGroup newHomotypicalGroup;
	
	public ChangeHomotypicGroupOperation(String label, IUndoContext undoContext, 
			Taxon taxon, Synonym synonym, HomotypicalGroup newHomotypicalGroup) {
		super(label, undoContext, taxon);
		
		this.synonym = synonym;
		if(synonym == null){
			throw new IllegalArgumentException(
					"A null synonym was provided.");
		}
		
		this.oldHomotypicalGroup = synonym.getHomotypicGroup();
		this.newHomotypicalGroup = newHomotypicalGroup;
	}


	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Get synonym name
		TaxonNameBase synonymName = this.synonym.getName();
		
		// TODO pass in homotypical group's taxon in case we are dragging from one editor to another
		
		// Switch groups
		oldHomotypicalGroup.removeTypifiedName(synonymName);
		newHomotypicalGroup.addTypifiedName(synonymName);		
		
		// Redraw editor if it exists
		return redrawOpenEditor();
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Get synonym name
		TaxonNameBase synonymName = this.synonym.getName();
		if(synonymName == null){
			// TODO
		}
		
		// TODO pass in homotypical group's taxon in case we are dragging from one editor to another
		
		// Switch groups
		newHomotypicalGroup.removeTypifiedName(synonymName);
		oldHomotypicalGroup.addTypifiedName(synonymName);		
		
		// Redraw editor if it exists
		return redrawOpenEditor();
	}
}
