/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/**
 * @author p.ciardelli
 * @created 15.01.2009
 * @version 1.0
 */
public class ChangeMisappliedNameToSynonymOperation extends
		AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(ChangeMisappliedNameToSynonymOperation.class);
	
	private Taxon misappliedName;
	private HomotypicalGroup homotypicalGroup;

	public ChangeMisappliedNameToSynonymOperation(String label,
			IUndoContext undoContext, Taxon taxon, Taxon misappliedName,
			HomotypicalGroup homotypicalGroup) {
		super(label, undoContext, taxon);
		
		this.misappliedName = misappliedName;
		this.homotypicalGroup = homotypicalGroup;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Create new synonym using misapplication's name
		TaxonNameBase synonymName = misappliedName.getName();
		Synonym synonym = Synonym.NewInstance(synonymName, taxon.getSec());
		
		// Remove misapplied name relation from taxon
		taxon.removeTaxon(misappliedName, TaxonRelationshipType.MISAPPLIED_NAME_FOR());

		// Add synonym to taxon
//		if (homotypicalGroup.equals(taxon.getHomotypicGroup())) {
//			
//			// Note: synonymName is implicitly added to taxon's homotypical group
//			taxon.addSynonym(synonym, SynonymRelationshipType.SYNONYM_OF() .HOMOTYPIC_SYNONYM_OF());	
//		} else {
//			taxon.addSynonym(synonym, SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF());
//			
//			// Add synonym to homotypical group
//			homotypicalGroup.addTypifiedName(synonymName);
//		}

		taxon.addSynonym(synonym, SynonymRelationshipType.SYNONYM_OF());
		
		// Add synonym to homotypical group
		homotypicalGroup.addTypifiedName(synonymName);
		
		return redrawOpenEditor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}
}
