/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/**
 * @author p.ciardelli
 * @created 15.01.2009
 * @version 1.0
 */
public class ChangeSynonymToTaxonOperation extends AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToTaxonOperation.class);
	private Synonym synonym;

	public ChangeSynonymToTaxonOperation(String text, IUndoContext undoContext,
			Taxon taxon, Synonym synonym) {
		super(text, undoContext, taxon);
		
		this.synonym = synonym;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Remove synonym from taxon
		taxon.removeSynonym(synonym);
		
		// Create new taxon with synonym name
		Taxon newTaxon = Taxon.NewInstance(synonym.getName(), synonym.getSec());
				
		// Add new taxon to session data repository
		taxon.setTaxonomicParent(null, null, null);
		CdmSessionDataRepository.getDefault().addTaxon(newTaxon);
		
		// Open editor for new taxon
		try {
			UiUtil.openTaxonEditor(newTaxon);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		
		return redrawOpenEditor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}
}
