/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.name.CdmParserController;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/**
 * @author p.ciardelli
 * @created 16.01.2009
 * @version 1.0
 */
public class CreateSynonymInNewGroupOperation extends AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(CreateSynonymInNewGroupOperation.class);
	
	private String newSynonymText;
	private SynonymRelationship newSynonymRelationship;

	public CreateSynonymInNewGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, String synonymText) {
		super(label, undoContext, taxon);
		
		this.newSynonymText = synonymText;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Create synonym name using newSynonymText
		TaxonNameBase synonymName = CdmParserController.parseFullReference(newSynonymText, null, null);
		
		// Add name to new homotypic group
		HomotypicalGroup.NewInstance().addTypifiedName(synonymName);
		
		// Create a new synonym for the taxon
		newSynonymRelationship = taxon.addHeterotypicSynonymName(synonymName);
				
		return redrawOpenEditor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Remove the synonym 
		taxon.removeSynonymRelation(newSynonymRelationship);
		newSynonymRelationship = null;
		return null;
	}
}
