/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.propertysheet.name;

import java.util.Collection;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.propertysheet.CollectionPropertySource;

public class NameRelationsPropertySource extends CollectionPropertySource {
	private static final Logger logger = Logger
			.getLogger(NameRelationsPropertySource.class);
	
	private TaxonNameBase name;
	private boolean isZoological = false;

	public NameRelationsPropertySource(TaxonNameBase name, Collection collection) {
		super(collection);
		this.name = name;
		if (name instanceof ZoologicalName) {
			isZoological = true;
		}
	}
	
	@Override
	protected String getItemDisplayName(Object item) {
		String itemDisplayName = "";
		
		if (item instanceof NameRelationship) {
			
			NameRelationship nameRelation = (NameRelationship) item;
			
			if (nameRelation.getFromName() instanceof ZoologicalName &&
					nameRelation.getToName() instanceof ZoologicalName) {
				isZoological = true;	
			}
			
			if (nameRelation.getType() != null) {
				
				// Is the problem the type ...
				NameRelationshipType type = nameRelation.getType();
				
				// or the label? LABEL
				String label = UiUtil.getNameRelationLabelType(type);
				if (isZoological && 
						nameRelation.getType().equals(NameRelationshipType.BASIONYM())) {
					label = "original combination";
				}
				itemDisplayName = label;
			} else {
				itemDisplayName = "Relation has no type";
			}
		}
		return itemDisplayName;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id instanceof NameRelationship) {
			TaxonNameBase relatedName;
			String nameValue;
			if (((NameRelationship) id).getFromName().equals(name)) {
				relatedName = ((NameRelationship) id).getToName();
				nameValue = relatedName.getTitleCache();
			} else {
				relatedName = ((NameRelationship) id).getFromName();
				nameValue = "to " + relatedName.getTitleCache();
			}
			return nameValue;
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		// Fields not editable in property sheet view
	}

	@Override
	public String toString() {
		// "Name Relations" header has no value
		return "";
	}
}