/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.propertysheet.CollectionPropertySource;

/**
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class NomStatusPropertySource extends CollectionPropertySource {
	private static final Logger logger = Logger.
			getLogger(NomStatusPropertySource.class);

	public NomStatusPropertySource(Collection collection) {
		super(collection);
	}

	static NomenclaturalStatusType[] nomStatusTypes = null;
	static String[] P_NOMSTATUS_MENU = null;
	private static void populateNomStatusTypes() {
		
		// Get terms from nom status vocabulary
		List<NomenclaturalStatusType> nomStatusTypesList = new ArrayList<NomenclaturalStatusType>();
		nomStatusTypesList.addAll(
				CdmSessionDataRepository.getDefault().getNomStatii().getTerms());
				
		// Populate nom status type menu labels 
		List<String> nomStatusTypesMenuList = new ArrayList<String>();
		for (NomenclaturalStatusType nomStatusType : nomStatusTypesList) {
			nomStatusTypesMenuList.add(nomStatusType.getLabel());
		}
		
		// Add an empty element for "nothing selected" in the rank drop-down 
		nomStatusTypesList.add(0, null);
		nomStatusTypesMenuList.add(0, "");
		
		// Convert rank lists to array
		nomStatusTypes = nomStatusTypesList.toArray(new NomenclaturalStatusType[nomStatusTypesList.size()]);
		P_NOMSTATUS_MENU = nomStatusTypesMenuList.toArray(new String[nomStatusTypesMenuList.size()]); 	
	}		
	

	@Override
	protected String getItemDisplayName(Object item) {
		// No display names on individual nom statii
		return "";
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id instanceof NomenclaturalStatus) {
			NomenclaturalStatus nomenclaturalStatus = (NomenclaturalStatus) id;
			NomenclaturalStatusType statusType = nomenclaturalStatus.getType(); 
			if (statusType != null) {
				return statusType.getLabel();
			}
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		// Fields not editable in property sheet view	
	}

	@Override
	public String toString() {
		// "Nom. Status" header has no value
		return "";
	}
}